﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace BasicMaze
{
    /// <summary>
    /// Means for storing the properties of a Maze cell.
    /// </summary>
    public class Cell
    {
        //X & Y coordinates of the Cell
        //Alternatively, a Cell could have a Position; I wasn't thinking in those terms
        //when creating the maze-generator.
        private int _positionX;
        private int _positionY;

        private int _distanceFromStart;
        /// <summary>
        /// Cell's distance from the position the Maze began to be created from.
        /// </summary>
        public int DistanceFromStart
        {
            get { return _distanceFromStart; }
            set { _distanceFromStart = value; }
        }

        //State of wall presence in each direction; "true" means direction walled
        private bool[] _walls = new bool[4];

        /// <summary>
        /// Create a new Cell.
        /// </summary>
        /// <param name="xPos">X-coordinate of Cell</param>
        /// <param name="yPos">Y-coordinate of Cell</param>
        public Cell(int xPos, int yPos)
        {
            _positionX = xPos;
            _positionY = yPos;
            //all cells start walled up
            for (int i = 0; i < _walls.Length; i++)
            {
                _walls[i] = true;            
            }
        }

        /// <summary>
        /// Set that a particular path out of a Cell is free.
        /// </summary>
        /// <param name="pathKey">Direction to be opened.</param>
        public void setOpenPath(int pathKey)
        {
            _walls[pathKey] = false;        
        }

        /// <summary>
        /// Check if a potential path out of a Cell is walled up.
        /// </summary>
        /// <param name="pathKey">Direction to check.</param>
        /// <returns>"True" if there is a wall, else "False"</returns>
        public bool isWalled(int pathKey)
        {
            return _walls[pathKey];                    
        }

        /// <summary>
        /// See if there are any unwalled directions - if not, the Cell has not yet been visited
        /// by the Maze generator.
        /// </summary>
        /// <returns>"True" if there are unwalled directions, else "False"</returns>
        public bool anyOpenPaths()
        { 
            for (int i=0; i<_walls.Length; i++)        
            {
                //if any path is open
                if (!_walls[i])
                {
                    return true;                
                }
            }
            return false;
        }

        //GAME CELL PROPERTIES
        //I'd like to have these in a subclass, but I couldn't get it to work.

        private bool _breadcrumb = false;
        /// <summary>
        /// Put down a marker.
        /// </summary>
        public bool Breadcrumb
        {
            get { return _breadcrumb; }
            set { _breadcrumb = value; }
        }

        private bool _playerMarker = false;
        /// <summary>
        /// Mark a player's presence.
        /// </summary>
        public bool PlayerMarker
        {
            get { return _playerMarker; }
            set { _playerMarker = value; }
        }

        private bool _enemyMarker = false;
        /// <summary>
        /// Mark an enemy's presence (not implemented).
        /// </summary>
        public bool EnemyMarker
        {
            get { return _enemyMarker; }
            set { _enemyMarker = value; }
        }
    }
}
