﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Maize
{
    /// <summary>
    /// Dialog for specifying dimensions for a new maze.
    /// </summary>
    public partial class NewMazeForm : Form
    {
        private int _horizontalSize;
        /// <summary>
        /// The horizontal/Y length of the new maze, always 2 or greater.
        /// </summary>
        public int HorizontalSize
        {
            get { return _horizontalSize; }
            set 
            {
                if (value > 300)
                {
                    _horizontalSize = 300;
                }
                else if (value > 1)
                {
                    _horizontalSize = value;
                }
                else
                {
                    _horizontalSize = 2;
                }
            }
        }

        private int _verticalSize;
        /// <summary>
        /// The vertical/X length of the maze, always 2 or greater.
        /// </summary>
        public int VerticalSize
        {
            get { return _verticalSize; }
            set
            {
                if (value > 300)
                {
                    _horizontalSize = 300;
                }       
                else if (value > 1)
                {
                    _verticalSize = value;
                }
                else
                {
                    _verticalSize = 2;
                }
             }
        }

        /// <summary>
        /// Creates the dialog for specifying the dimensions of a new maze.
        /// </summary>
        /// <param name="xSizeDefault">Vertical/X default length</param>
        /// <param name="ySizeDefault">Horizontal/Y default length</param>
        public NewMazeForm(int xSizeDefault, int ySizeDefault)
        {
            InitializeComponent();
            //populate text boxes
            horizontalSizeMaskedTextBox.Text = Convert.ToString(ySizeDefault);
            verticalSizeMaskedTextBox.Text = Convert.ToString(xSizeDefault);
            //helps the sizes be "select-all" by default (faster entry)
            verticalSizeMaskedTextBox.GotFocus += verticalSizeMaskedTextBox_GotFocus;
            horizontalSizeMaskedTextBox.GotFocus += horizontalSizeMaskedTextBox_GotFocus;
        }

        //Close the dialog.
        private void cancelButton_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        //Accept the values and close the dialog (with OK result)
        private void okButton_Click(object sender, EventArgs e)
        {
            if (!horizontalSizeMaskedTextBox.Text.Equals(string.Empty))
            {
                HorizontalSize = Convert.ToInt16(horizontalSizeMaskedTextBox.Text);
            }
            else
            {
                HorizontalSize = 2;            
            }

            if (!verticalSizeMaskedTextBox.Text.Equals(string.Empty))
            {
                VerticalSize = Convert.ToInt16(verticalSizeMaskedTextBox.Text);
            }
            else 
            {
                VerticalSize = 2;            
            }

            this.Close();
        }

        //select everything in the text boxes
        private void horizontalSizeSelect()
        {
            horizontalSizeMaskedTextBox.SelectionStart = 0;
            horizontalSizeMaskedTextBox.SelectionLength = horizontalSizeMaskedTextBox.Text.Length;        
        }

        private void verticalSizeSelect()
        {
            verticalSizeMaskedTextBox.SelectionStart = 0;
            verticalSizeMaskedTextBox.SelectionLength = verticalSizeMaskedTextBox.Text.Length;
        }

        //bring cursor to the first text box when dialog is shown
        private void NewMazeForm_Shown(object sender, EventArgs e)
        {
            horizontalSizeMaskedTextBox.Select();
        }

        //select everything on tab-in:
        private void horizontalSizeMaskedTextBox_GotFocus(object sender, EventArgs e)
        {
            horizontalSizeSelect();
        }

        private void verticalSizeMaskedTextBox_GotFocus(object sender, EventArgs e)
        {
            verticalSizeSelect();
        }

        //select everything on click-in:
        private void horizontalSizeMaskedTextBox_Click(object sender, EventArgs e)
        {
            horizontalSizeSelect();
        }

        private void verticalSizeMaskedTextBox_Click(object sender, EventArgs e)
        {
            verticalSizeSelect();
        }
    }
}
