﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using BasicMaze;

namespace Maize
{
    public partial class OldMaizeGameForm : Form
    {
        private Cell[,] _mazeCells;
        private int _xStartPos;
        private int _yStartPos;
        private int _xFinishPos;
        private int _yFinishPos;
        private int _xSetSize = 15; //defaults
        private int _ySetSize = 15;

        List<string> _mazeText;
        MazeDrawing _myMazeDrawing;

        public OldMaizeGameForm()
        {
            InitializeComponent();
            SummonMaze(_xSetSize, _ySetSize);
        }

        private void SummonMaze(int xSize, int ySize)
        {
            Maze myMaze = new Maze(xSize, ySize);
            _xStartPos = myMaze.getXStart();
            _yStartPos = myMaze.getYStart();
            _xFinishPos = myMaze.getXFinish();
            _yFinishPos = myMaze.getYFinish();
            
            //get the cells
            _mazeCells = myMaze.getCells();

            //fire up maze drawing class
            _myMazeDrawing = new MazeDrawing(_mazeCells, xSize, ySize, _xStartPos, _yStartPos, _xFinishPos, _yFinishPos);

            //draw the text version maze
            //TODO... move to export routine
            DrawAsText(xSize, ySize);

            //TODO: Graphical version :-)
        }

        private void DrawAsText(int xLength, int yLength)
        {
            testingTextBox.Clear();

            _mazeText = _myMazeDrawing.DrawAsText();

            //fill text box
            for (int i=0; i<_mazeText.Count-1;i++)
            {
                testingTextBox.AppendText(_mazeText.ElementAt(i) + "\r\n");
            }
            testingTextBox.AppendText(_mazeText.ElementAt(_mazeText.Count - 1));

        }

        /// <summary>
        /// Show the AboutBox.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void aboutMaizeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MaizeAboutBox aboutBox = new MaizeAboutBox();
            aboutBox.ShowDialog();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void newMazeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NewMazeForm newMaze = new NewMazeForm(_xSetSize, _ySetSize);
            newMaze.ShowDialog();
            //grab vars from form if applicable and build maze
            if (newMaze.DialogResult == DialogResult.OK)
            {
                _xSetSize = newMaze.VerticalSize;
                _ySetSize = newMaze.HorizontalSize;
                SummonMaze(_xSetSize, _ySetSize);
                _myMazeDrawing = new MazeDrawing(_mazeCells, _xSetSize, _ySetSize, _xStartPos, _yStartPos, _xFinishPos, _yFinishPos);
                gamePanel.Invalidate();
            }
        }

        /*
         Robust Programming
            The following conditions may cause an exception:
            The file exists and is read-only (IOException).
            The path name may be too long (PathTooLongException).
            The disk may be full (IOException).
         */


        //draw the maze
        private void gamePanel_Paint(object sender, PaintEventArgs e)
        {
            decimal xScaleMax = gamePanel.Size.Height / _xSetSize;
            decimal yScaleMax = gamePanel.Size.Width / _ySetSize;
            if (yScaleMax < xScaleMax)
            {
                xScaleMax = yScaleMax;            
            }
            _myMazeDrawing.CellScale = Convert.ToInt32(xScaleMax);
            _myMazeDrawing.DrawAsGraphics(e);
        }

        //http://msdn.microsoft.com/en-us/library/sfezx97z.aspx
        private void textToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileDialog.Filter = "Unicode text file (*.txt)|*.txt";
            saveFileDialog.Title = "Save maze as text";

            saveFileDialog.ShowDialog();
            if (saveFileDialog.FileName != String.Empty)
            {
                try
                {
                    // Write the string to a file.
                    //http://msdn.microsoft.com/en-us/library/aa287548(v=vs.71).aspx
                    System.IO.StreamWriter outFile = new System.IO.StreamWriter(saveFileDialog.FileName);

                    foreach (string line in _mazeText)
                    {
                        outFile.WriteLine(line);
                    }
                    outFile.Close();
                }
                catch (System.IO.IOException ioE)
                {
                    MessageBox.Show(ioE.Message);
                }
            }
        }

        private void imageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //"JPeg Image|*.jpg|Bitmap Image|*.bmp|Gif Image|*.gif";
            saveFileDialog.Filter = "PNG Image (*.png)|*.png|Windows Bitmap (*.bmp)|*.bmp|GIF Image (*.gif)|*.gif|JPEG Image (*.jpg)|*.jpg";
            saveFileDialog.Title = "Save maze as image";

            saveFileDialog.ShowDialog();
            if (saveFileDialog.FileName != String.Empty)
            {
                try
                {
                    System.Drawing.Bitmap mazeImage = new System.Drawing.Bitmap(gamePanel.Width, gamePanel.Height);
                    gamePanel.DrawToBitmap(mazeImage, gamePanel.ClientRectangle);

                    // Saves the Image via a FileStream created by the OpenFile method.
                    //http://msdn.microsoft.com/en-us/library/sfezx97z.aspx
                    System.IO.FileStream fs =
                       (System.IO.FileStream)saveFileDialog.OpenFile();
                    // Saves the Image in the appropriate ImageFormat based upon the
                    // File type selected in the dialog box.
                    // NOTE that the FilterIndex property is one-based.
                    switch (saveFileDialog.FilterIndex)
                    {
                        case 1:
                            mazeImage.Save(fs,
                               System.Drawing.Imaging.ImageFormat.Png);
                            break;

                        case 2:
                            mazeImage.Save(fs,
                               System.Drawing.Imaging.ImageFormat.Bmp);
                            break;

                        case 3:
                            mazeImage.Save(fs,
                               System.Drawing.Imaging.ImageFormat.Gif);
                            break;

                        case 4:
                            mazeImage.Save(fs,
                               System.Drawing.Imaging.ImageFormat.Jpeg);
                            break;
                    }

                    fs.Close();
                }
                catch (System.IO.IOException ioE)
                {
                    MessageBox.Show(ioE.Message);
                }
            }
        }

   
    }
}
