﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using BasicMaze;

namespace Maize
{
    public partial class MaizeGameForm : Form
    {
        private Cell[,] _mazeCells;
        private int _xStartPos;
        private int _yStartPos;
        private int _xFinishPos;
        private int _yFinishPos;
        private int _playerX;
        private int _playerY;
        private int _xSetSize = 10; //defaults
        private int _ySetSize = 10;
        private bool _winning = false;

        List<string> _mazeText;
        MazeDrawing _myMazeDrawing;

        public MaizeGameForm()
        {
            InitializeComponent();
            this.WindowState = FormWindowState.Maximized;
            SummonMaze(_xSetSize, _ySetSize);
        }

        private void SummonMaze(int xSize, int ySize)
        {
            Maze myMaze = new Maze(xSize, ySize);
            _xStartPos = myMaze.getXStart();
            _yStartPos = myMaze.getYStart();
            _xFinishPos = myMaze.getXFinish();
            _yFinishPos = myMaze.getYFinish();

            //get cells
            _mazeCells = myMaze.getCells();

            //initialize game pieces
            _playerX = _xStartPos;
            _playerY = _yStartPos;
            _mazeCells[_xStartPos, _yStartPos].PlayerMarker = true;

            //fire up maze drawing class
            _myMazeDrawing = new MazeDrawing(_mazeCells, xSize, ySize, _xStartPos, _yStartPos, _xFinishPos, _yFinishPos);
        }

        /// <summary>
        /// Show the AboutBox.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void aboutMaizeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MaizeAboutBox aboutBox = new MaizeAboutBox();
            aboutBox.ShowDialog();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void newMazeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NewMazeForm newMaze = new NewMazeForm(_xSetSize, _ySetSize);
            newMaze.ShowDialog();
            //grab vars from form if applicable and build maze
            if (newMaze.DialogResult == DialogResult.OK)
            {
                _xSetSize = newMaze.VerticalSize;
                _ySetSize = newMaze.HorizontalSize;
                SummonMaze(_xSetSize, _ySetSize);
                _myMazeDrawing = new MazeDrawing(_mazeCells, _xSetSize, _ySetSize, _xStartPos, _yStartPos, _xFinishPos, _yFinishPos);
                gamePanel.Invalidate();
            }
        }

 



        //draw the maze
        private void gamePanel_Paint(object sender, PaintEventArgs e)
        {
            if (_winning)
            {
                return;                
            }
            else
            {
                //find max scale for maze
                decimal xScaleMax = (gamePanel.Size.Height - 1) / _xSetSize;
                decimal yScaleMax = (gamePanel.Size.Width - 1) / _ySetSize;
                if (yScaleMax < xScaleMax)
                {
                    xScaleMax = yScaleMax;
                }
                _myMazeDrawing.CellScale = Convert.ToInt32(xScaleMax);

                //left indent 1/2 dif between panel width and maze width
                int leftIndent = (int)((gamePanel.Size.Width - (xScaleMax * _ySetSize)) / 2);
                //top indent 1/2 dif between panel height and maze height
                int topIndent = (int)((gamePanel.Size.Height - (xScaleMax * _xSetSize)) / 2);

                //testing
                //label1.Text = this.Size.Height.ToString();
                //label2.Text = this.Size.Width.ToString();
                //label4.Text = leftIndent.ToString();
                //label3.Text = topIndent.ToString();

                if (leftIndent > 0)
                {
                    _myMazeDrawing.LrIndent = leftIndent;
                }
                else
                {
                    _myMazeDrawing.LrIndent = 0;
                }

                if (topIndent > 0)
                {
                    _myMazeDrawing.UdIndent = topIndent;
                }
                else
                {
                    _myMazeDrawing.UdIndent = 0;
                }

                _myMazeDrawing.DrawAsGraphics(e);
            }
        }

        /*
         Robust Programming for saving
            The following conditions may cause an exception:
            The file exists and is read-only (IOException).
            The path name may be too long (PathTooLongException).
            The disk may be full (IOException).
         */

        //http://msdn.microsoft.com/en-us/library/sfezx97z.aspx
        private void textToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileDialog.FileName = string.Empty;
            saveFileDialog.Filter = "Unicode text file (*.txt)|*.txt";
            saveFileDialog.Title = "Save maze as text";

            saveFileDialog.ShowDialog();
            if (saveFileDialog.FileName != string.Empty)
            {
                try
                {
                    // Write the string to a file.
                    //http://msdn.microsoft.com/en-us/library/aa287548(v=vs.71).aspx
                    System.IO.StreamWriter outFile = new System.IO.StreamWriter(saveFileDialog.FileName);

                    _mazeText = _myMazeDrawing.DrawAsText();

                    foreach (string line in _mazeText)
                    {
                        outFile.WriteLine(line);
                    }
                    outFile.Close();
                }
                catch (System.IO.IOException ioE)
                {
                    MessageBox.Show(ioE.Message);
                }
            }
        }

        private void imageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //Major TODO: SVG creation :-)
            saveFileDialog.FileName = string.Empty;
            saveFileDialog.Filter = "Portable Network Graphics (*.png)|*.png|Windows Bitmap (*.bmp)|*.bmp|Graphics Interchange Format (*.gif)|*.gif|JPEG Image (*.jpg)|*.jpg";
            saveFileDialog.Title = "Save maze as image";

            saveFileDialog.ShowDialog();
            if (saveFileDialog.FileName != string.Empty)
            {
                try
                {
                    
                    // Saves the Image via a FileStream created by the OpenFile method.
                    //http://msdn.microsoft.com/en-us/library/sfezx97z.aspx
                    System.IO.FileStream fs =
                       (System.IO.FileStream)saveFileDialog.OpenFile();

                    //scale, margin
                    //later versions will let users pick these in some sort of dialog
                    
                    // Saves the Image in the appropriate ImageFormat based upon the
                    // File type selected in the dialog box.
                    // NOTE that the FilterIndex property is one-based.
                    switch (saveFileDialog.FilterIndex)
                    {
                        //case 1:
                          //{
                          //    MessageBox.Show("SVG coming soon."); //maybe
                          //    break;
                          //}

                        case 1:
                            {
                                System.Drawing.Bitmap mazeImage = _myMazeDrawing.DrawAsImage(25, 2);
                                mazeImage.Save(fs,
                                   System.Drawing.Imaging.ImageFormat.Png);
                                break;
                            }

                        case 2:
                            {
                                System.Drawing.Bitmap mazeImage = _myMazeDrawing.DrawAsImage(25, 2);
                                mazeImage.Save(fs,
                                   System.Drawing.Imaging.ImageFormat.Bmp);
                                break;
                            }

                        case 3:
                            {
                                System.Drawing.Bitmap mazeImage = _myMazeDrawing.DrawAsImage(25, 2);
                                mazeImage.Save(fs,
                                   System.Drawing.Imaging.ImageFormat.Gif);
                                break;
                            }

                        case 4:
                            {
                                System.Drawing.Bitmap mazeImage = _myMazeDrawing.DrawAsImage(25, 2);
                                mazeImage.Save(fs,
                                   System.Drawing.Imaging.ImageFormat.Jpeg);
                                break;
                            }
                    }

                    fs.Close();
                }
                catch (System.IO.IOException ioE)
                {
                    MessageBox.Show(ioE.Message);
                }
            }
        }

        //http://blogs.msdn.com/b/alejacma/archive/2008/11/20/controls-won-t-get-resized-once-the-nesting-hierarchy-of-windows-exceeds-a-certain-depth-x64.aspx
        private void MaizeGameForm_Resize(object sender, EventArgs e)
        {
            //Did this because I thought the maze resize wasn't getting called
            //but it was actually my not resetting indents to 0
            if (gamePanel.Handle != null)
            {
                BeginInvoke(new MethodInvoker(ChangeSize));
            }

            //base.OnResize(e);
            
        }

        protected void ChangeSize()
        {
            gamePanel.Invalidate();
            gamePanel.Update();
        }

        private void restartMazeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //move the player back to the starting point:
            //take player away from where he is
            _mazeCells[_playerX, _playerY].PlayerMarker = false;
            //set his coords to the starting position
            _playerX = _xStartPos;
            _playerY = _yStartPos;
            //set that the player is there
            _mazeCells[_playerX, _playerY].PlayerMarker = true;

            //zap all the breadcrumbs
            for (int i = 0; i < _xSetSize; i++)
            {
                for (int j = 0; j < _ySetSize; j++)
                {
                    _mazeCells[i, j].Breadcrumb = false;
                }
            }

            gamePanel.Invalidate();
            gamePanel.Update();
        }

        //pressing a key
        private void MaizeGameForm_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyData.ToString())
            {
                case "Left":
                    if (!_mazeCells[_playerX, _playerY].isWalled((int) Directions.W))
                    {
                        _mazeCells[_playerX, _playerY].PlayerMarker = false;
                        if (leaveBreadcrumbsMenuItem.Checked)
                        {
                            _mazeCells[_playerX, _playerY].Breadcrumb = true;                        
                        }
                        _playerY -= 1;
                        _mazeCells[_playerX, _playerY].PlayerMarker = true;
                    }
                    break;

                case "Right":
                    if (!_mazeCells[_playerX, _playerY].isWalled((int)Directions.E))
                    {
                        _mazeCells[_playerX, _playerY].PlayerMarker = false;
                        if (leaveBreadcrumbsMenuItem.Checked)
                        {
                            _mazeCells[_playerX, _playerY].Breadcrumb = true;
                        }
                        _playerY += 1;
                        _mazeCells[_playerX, _playerY].PlayerMarker = true;
                    }
                    break;

                case "Up":
                    if (!_mazeCells[_playerX, _playerY].isWalled((int)Directions.N))
                    {
                        _mazeCells[_playerX, _playerY].PlayerMarker = false;
                        if (leaveBreadcrumbsMenuItem.Checked)
                        {
                            _mazeCells[_playerX, _playerY].Breadcrumb = true;
                        }
                        _playerX -= 1;
                        _mazeCells[_playerX, _playerY].PlayerMarker = true;
                    }
                    break;

                case "Down":
                    if (!_mazeCells[_playerX, _playerY].isWalled((int)Directions.S))
                    {
                        _mazeCells[_playerX, _playerY].PlayerMarker = false;
                        if (leaveBreadcrumbsMenuItem.Checked)
                        {
                            _mazeCells[_playerX, _playerY].Breadcrumb = true;
                        }
                        _playerX += 1;
                        _mazeCells[_playerX, _playerY].PlayerMarker = true;
                    }
                    break;

                case "Space":
                    if (screenTimer.Enabled)
                    {
                        screenTimer.Stop();
                        resetGameField(sender, e);
                    }
                    break;

            }

            if ((_playerX == _xFinishPos) && (_playerY == _yFinishPos))
            {
                
                //TODO: change to graphics
                _winning = true;
                outerPanel.BackColor = Color.LightGoldenrodYellow;
                gamePanel.Hide();
                backgroundImageSetter();
                //make sure the images loaded
                //Application.DoEvents();
                //outerPanel.BringToFront();
                //change to graphics
                //"press space to continue"
                
                //TODO: wait 5 seconds
                screenTimer.Interval = 5000;
                screenTimer.Tick += new EventHandler(resetGameField);
                screenTimer.Start();
            
               
            }

            gamePanel.Invalidate();
            gamePanel.Update();

        }

        private void resetGameField(object sender, EventArgs e)
        {
            screenTimer.Stop();
            outerPanel.BackgroundImage = null;
            outerPanel.BackColor = Color.Black;
            _winning = false;
            gamePanel.Show();
                
            _xSetSize += (_xSetSize / 2);
            _ySetSize += (_ySetSize / 2);
            if (_xSetSize > 300)
            {
                _xSetSize = 300;
            }
            if (_ySetSize > 300)
            {
                _ySetSize = 300;
            }
            SummonMaze(_xSetSize, _ySetSize);
            

            gamePanel.Invalidate();
            gamePanel.Update();
        
        }


        private void backgroundImageSetter()
        {
            Random random = new Random();
            int imagePick = random.Next(5);

            
            

            /*switch (imagePick)
            { 
                case 0:
                    outerPanel.BackgroundImage = Properties.Resources.Corncobs;
                    break;

                case 1:
                    outerPanel.BackgroundImage = Properties.Resources._768px_CornWaterTower;
                    break;

                case 2:
                    outerPanel.BackgroundImage = Properties.Resources._1024px_Field__corn__Liechtenstein__Mountains__Alps__Vaduz__sky__clouds__landscape;
                    break;

                case 3:
                    outerPanel.BackgroundImage = Properties.Resources._1024px_Ab_food_06;
                    break;

                case 4:
                    outerPanel.BackgroundImage = Properties.Resources._1024px_Männliche_Blüte_einer_Maispflanze_2009_08_19;
                    break;
            
            }*/

            outerPanel.Invalidate();
            outerPanel.Update();

            

            
                           
        
        }

        private void outerPanel_Paint(object sender, PaintEventArgs e)
        {
            if (_winning)
            {
                Graphics g = e.Graphics;
                SolidBrush myBrush = new SolidBrush(Color.Black);
                Pen myPen = new Pen(myBrush, 1);
                g.DrawRectangle(myPen, 0, 0, 180, 180);
            }
            else
            {
                return;            
            }

        }

           
    }
}
